require 'api_consumer'

module EME
  class TeraGameData < APIConsumer
    def self.guilds_by_leader_account(leader_master_account_id, server_name, conn = connection, opts = {})
      return do_request("/guilds/owned_by_user/#{server_name}/#{leader_master_account_id}", conn, opts)
    end

    def self.guild(guild_id, server_name, conn = connection, opts = {})
      return do_request("/guilds/#{server_name}/#{guild_id}", conn, opts)
    end
    
    def self.guild_of_character(character_id, server_name, conn = connection, opts={})
      return do_request("/guilds/of_character/#{server_name}/#{character_id}", conn, opts)
    end
    
    def self.search_characters_by_name(character_name, server_name, conn = connection, opt={})
      #body = {:name => character_name, :server_id => server_abbr(server_name)}.to_json
      return do_request("/characters/search?name=#{character_name}&server_id=#{server_id(server_name)}", conn)
    end
    
    def self.search_characters_by_game_account(game_account_id, server_name, conn = connection, opt={})
      #body = {:game_account_id => game_account_id, :server_id => server_id(server_name)}.to_json
      return do_request("/characters/search?game_account_id=#{game_account_id}&server_id=#{server_id(server_name)}", conn)
    end
    
    def self.servers(conn_flag = :normal, reload = false)
      errors = { "404" => "wat", "500" => "Server Error" }
      return do_request("/servers", connection(conn_flag), {:errors => errors, :key => "server-list", :ttl => 300})
    end
    
    def self.server_abbr(server_name)
      return PRESET_ABBR[server_name] if PRESET_ABBR[server_name]
      tmp = server_name.split(' ')
      return(tmp[0][0] + tmp[1][0]).upcase if tmp.length > 1
      (tmp[0][0] + tmp[0][1]).upcase
    end
    PRESET_ABBR = {"Highwatch" => 'HW', "Lake of Tears" => "LT", "Valley of Titans" => "VT", "New Pora" => "PA"}.freeze
    
    RACE_OPTIONS = [["Human", 0], ["High Elf", 1], ["Aman", 2], ["Castanic", 3], ["Elin", 4], ["Popori", 4], ["Baraka", 5]]
    GENDER_OPTIONS = [["Male", 0], ["Female", 1]]
    
    def self.display_race(race_id, gender_id)
      return RACES[race_id][gender_id]
    end
    RACES = {
      0 => {0 => "Human", 1 => "Human"},
      1 => {0 => "High Elf", 1 => "High Elf"},
      2 => {0 => "Aman", 1 => "Aman"},
      3 => {0 => "Castanic", 1 => "Castanic"},
      4 => {0 => "Popori", 1 => "Elin"},
      5 => {0 => "Baraka", 1 => "Baraka"}
    }
    def self.display_class(class_id)
      return class_id if class_id >= CLASSES.count
      return CLASSES[class_id]
    end
    CLASSES = ["Warrior", "Lancer", "Slayer", "Berserker", "Sorcerer", "Archer", "Priest", "Mystic", "Reaper", "Gunner", "Brawler", "Ninja", "Valkyrie"]

    def self.server_id(server_name)
      server_list = servers
      server = server_list.select{|x| x["name"] == server_name || server_abbr(x["name"]) == server_name}[0]
      return server["server_id"] if server
      return nil
    end
  end
end
